#include "CorrespEdit.h"
#include "util.h"
#include <iostream>

using namespace std;

CorrespEdit::CorrespEdit(QWidget *parent)
	: QWidget(parent)
{
	ui.setupUi(this);
	setAttribute(Qt::WA_DeleteOnClose);		// Destroy the object when closing

	init();
}

CorrespEdit::~CorrespEdit()
{
	for (int i = 0; i < controls.size(); i++) {
		delete controls[i];
	}
}

void CorrespEdit::init()
{
	controls.clear();

	// IMPORTANT!
	// To add a new control, insert the line here!  That's it!
	controls.push_back(new ParamSelect(this, "Colormap Range Min", COLORMAP_RANGE_MIN));
	controls.push_back(new ParamSelect(this, "Colormap Range Max", COLORMAP_RANGE_MAX));
	controls.push_back(new ParamSelect(this, "# Source Samples", NUMBER_SAMPLES_SOURCE));
	controls.push_back(new ParamSelect(this, "# Target Samples", NUMBER_SAMPLES_TARGET));
	controls.push_back(new ParamSelect(this, "Feature Space Range", FEATURE_SPACE_RANGE));
	controls.push_back(new ParamSelect(this, "Feature Space Threshold", FEATURE_SPACE_THRESHOLD));
	controls.push_back(new ParamSelect(this, "Feature Space Max Matches", FEATURE_SPACE_MAX_MATCHES));
	controls.push_back(new ParamSelect(this, "Mean Shift Bandwidth", MEAN_SHIFT_BANDWIDTH));
	controls.push_back(new ParamSelect(this, "Min Cluster Size", MIN_CLUSTER_SIZE));
	controls.push_back(new ParamSelect(this, "Selected Cluster", CLUSTER_SELECT));
	controls.push_back(new ParamSelect(this, "Max Pair Distance", MAX_PAIR_DISTANCE));
	controls.push_back(new ParamSelect(this, "Min Verified Region Size", MIN_VERIFY_REGION_SIZE));
	controls.push_back(new ParamSelect(this, "Max # Clusters", MAX_NUM_CLUSTERS));
	controls.push_back(new ParamSelect(this, "Max # Verified Regions", MAX_VERIFIED_REGIONS));

	//////////////////////////////////////////////////////////////////////////
	for (int i = 0; i < controls.size(); i++) {
		connect(controls[i], SIGNAL(valueChanged(int, double)), this, SLOT(controlChanged(int, double)));
	}
}

void CorrespEdit::controlChanged(int id, double value) {
	emit valueChanged(id, value);
}

void CorrespEdit::on_ComputeFeaturesButton_clicked()
{
	emit valueChanged(COMPUTE_FEATURES, 1);
}

void CorrespEdit::on_ComputeMeanShiftButton_clicked()
{
	emit valueChanged(COMPUTE_MEAN_SHIFT, 1);
}

void CorrespEdit::on_ComputeTransformsButton_clicked()
{
	emit valueChanged(COMPUTE_TRANSFORMS, 1);
}

void CorrespEdit::on_SaveTransformsButton_clicked()
{
	emit valueChanged(TRANSFORMS_SAVE, 1);
}

void CorrespEdit::on_VerifyTransformationsButton_clicked()
{
	emit valueChanged(TRANSFORMS_VERIFY, 1);
}

void CorrespEdit::on_RefineClusterButton_clicked()
{
	emit valueChanged(REFINE_CLUSTER, 1);
}

void CorrespEdit::on_GraphCutRelabelButton_clicked()
{
	emit valueChanged(GRAPH_CUT_RELABEL, 1);
}

void CorrespEdit::on_FeatureViewerButton_clicked()
{
	emit valueChanged(FEATURE_VIEWER, 1);
}

void CorrespEdit::on_SampleCorrespButton_clicked()
{
	emit valueChanged(SAMPLE_CORRESP, 1);
}


ParamSelect* CorrespEdit::findControl(int id)
{
	for (int i = 0; i < controls.size(); i++) {
		if (controls[i]->getId() == id) {
			return controls[i];
		}
	}

	return NULL;
}

void CorrespEdit::initRange( int which, double minRange, double maxRange, double initValue )
{
	ParamSelect *control;
	if (control = findControl(which)) {
		control->initRange(minRange, maxRange, initValue);
	}
	else {
		cerr << "Warning in CorrespEdit::initRange(): Control " << which << " not found!" << endl;
	}
}
