#ifndef __CORRESPONDENCE_H__
#define __CORRESPONDENCE_H__

#include "PointList.h"

class SE3;
class ShapeModel;

class CorrespondenceList {
public:

	//////////////////////////////////////////////////////////////////////////
	// Access
	size_type size() const;										/**< The total size of points. */
	real3_type sourcePos( index_type i ) const;					/**< Retrieve the i^th source point. */
	real3_type targetPos( index_type i ) const;					/**< Retrieve the i^th target point. */

	//////////////////////////////////////////////////////////////////////////
	// Change
	void clear();												/**< Clear all correspondence pairs. */
	void resize( size_type size );								/**< Resize the list of correspondences. */
	void reserve( size_type size );								/**< Reserve space for the list of correspondences. */
	void add( real3_type sourcePt, real3_type targetPt,
		      index_type sourceIndex = 0, index_type targetIndex  = 0);	/**< Add a new correspondence pair, with indices. */
	void remove( idxList list );								/**< Remove a list of pairs. */

	void setSourcePos( index_type i, real3_type pt );
	void setTargetPos( index_type i, real3_type pt );

	//////////////////////////////////////////////////////////////////////////
	// Loading and saving
	bool load(std::string filename);
	bool save(std::string filename) const;

	void loadBinary( std::ifstream &in );
	void saveBinary( std::ofstream &out ) const;

	//////////////////////////////////////////////////////////////////////////
	// Performing ICP registration
	SE3 registerICP( ShapeModel &target, const SE3 &tf );
	void projectToShape( ShapeModel &target );

	//////////////////////////////////////////////////////////////////////////
	// Data
	PointList sourcePoints, targetPoints;						/**< Source and target point locations */
	idxList sourceIndex, targetIndex;							/**< Indices for the source and target points */
};

#endif
