#ifndef __COST_FUNCTIONS_H__
#define __COST_FUNCTIONS_H__

#include "types.h"

class SE3;
class ShapeModel;


// Data Functions
real_type data_PointToPoint( index_type sourceIndex, const SE3 &tf, ShapeModel &source, ShapeModel &target );
real_type data_PointToPlane( index_type sourceIndex, const SE3 &tf, ShapeModel &source, ShapeModel &target );
real_type data_Hybrid( index_type sourceIndex, const SE3 &tf, ShapeModel &source, ShapeModel &target );
real_type data_D6( index_type sourceIndex, const SE3 &tf, ShapeModel &source, ShapeModel &target );

// Smoothness Functions
real_type smooth_EdgeLength( index_type sourceIndex, const SE3 &sourceTf,
							 index_type targetIndex, const SE3 &targetTf,
							 ShapeModel &source, ShapeModel &target );


// Type definition for function pointers of data and smoothness functions
typedef real_type (*DataFcnType)( index_type, const SE3&, ShapeModel&, ShapeModel& );
typedef real_type (*SmoothFcnType)( index_type, const SE3&, index_type, const SE3&, ShapeModel&, ShapeModel& );

// Arrays of function pointers
typedef std::vector<DataFcnType> DataFcnArray;
typedef std::vector<SmoothFcnType> SmoothFcnArray;

// Evaluate data and smoothness costs
real_type evaluateDataCost( const std::vector<SE3> &assignedTf, ShapeModel &source, ShapeModel &target, DataFcnType dataFcn );
real_type evaluateSmoothCost( const std::vector<SE3> &assignedTf, ShapeModel &source, ShapeModel &target, SmoothFcnType smoothFcn );
real_type evaluateTotalCost( const std::vector<SE3> &assignedTf, ShapeModel &source, ShapeModel &target, DataFcnType dataFcn, SmoothFcnType smoothFcn );
data_type evaluateDataCost( const std::vector<SE3> &assignedTf, ShapeModel &source, ShapeModel &target, DataFcnArray dataFcn );
data_type evaluateSmoothCost( const std::vector<SE3> &assignedTf, ShapeModel &source, ShapeModel &target, SmoothFcnArray smoothFcn );
data_type evaluateTotalCost( const std::vector<SE3> &assignedTf, ShapeModel &source, ShapeModel &target, DataFcnArray dataFcn, SmoothFcnArray smoothFcn );

#endif
