#ifndef __SHAPE_SLICER_H__
#define __SHAPE_SLICER_H__

#include <QGLViewer/qglviewer.h>
#include <QMenu>
#include "types.h"

class Scene;
class ShapeModel;

class ShapeSlicer : public QGLViewer
{
	Q_OBJECT

public:
	ShapeSlicer(Scene *sc, ShapeModel* s);				/**< Constructor: initialize members here. */
	virtual ~ShapeSlicer();								/**< Destructor: destroy members here. */

public slots:
	// Load/Save Slicing result
	void loadSliceResult();
	void saveSliceResult();

	// Add current sliced mesh to scene
	void addToScene();

protected :
	virtual void draw();
	virtual void init();
	virtual void drawWithNames();
	void drawSelectionRectangle() const;
	virtual void mousePressEvent(QMouseEvent *e);
	virtual void mouseMoveEvent(QMouseEvent *e);
	virtual void mouseReleaseEvent(QMouseEvent *e);
	virtual void keyPressEvent(QKeyEvent *e);
	void endSelection(const QPoint&);

private:
	// Pointer to the entire scene
	Scene *scene;

	// Pointer to the shape
	ShapeModel *shape;
	RealColor shapeColor;

	//////////////////////////////////////////////////////////////////////////
	// Selection variables
	// Current rectangular selection
	QRect rectangle_;

	// Different selection modes
	enum SelectionMode { NONE, TOGGLE };
	SelectionMode selectionMode_;

	// Point select threshold
	const int pointSelect;

	// Selection
	idxList selected;								/**< Selected points */
	void addIdSelection(index_type id);
	void selectAll();

	//////////////////////////////////////////////////////////////////////////
	// Menu variables and functions
	void createContextMenu();

	QMenu* contextMenu;
	QAction *loadSliceResultAct;
	QAction *saveSliceResultAct;
	QAction *addToSceneAct;

};

#endif
