#ifndef __SPIN_IMAGE_H__
#define __SPIN_IMAGE_H__

#include "types.h"

/**
 * Apply the spin map to a point.
 */
real2_type SpinMap( const real3_type &p, const real3_type &n, const real3_type &x );
real_type GeometricConsistency( const real3_type &p_s1, const real3_type &n_s1,
							    const real3_type &p_s2, const real3_type &n_s2,
								const real3_type &p_t1, const real3_type &n_t1,
								const real3_type &p_t2, const real3_type &n_t2 );

class ShapeModel;
class CorrespondenceList;
real_type GeometricConsistency_Diameter( ShapeModel &source, ShapeModel &target, CorrespondenceList &cl );


class SpinImage
{
public:

	void accumulate( real2_type coord, real_type binSize );			/**< Accumulate the given value in the spin image. */
	int countBins() const;											/**< Count the number of non-empty bins. */

	void resize( int width );				/**< Allocate the space for the spin image. */
	void reset();							/**< Reset all the values of the spin image. */
	bool invalid() const;					/**< Check if there's no data in this spin image. */

	inline int width() const {
		return data.size();
	}

	inline real_type& operator()( int i, int j ) {
		return data[i][j];
	}
	inline const real_type& operator()( int i, int j ) const {
		return data[i][j];
	}

private:
	std::vector< std::vector<real_type> > data;
};


real_type ComputeSimilarity( const SpinImage &P, const SpinImage &Q, real_type lambda );


class SpinImageStack
{
public:

	inline size_type size() const {
		return data.size();
	}

	void resize( size_type n );

	inline SpinImage& operator[]( int i ) {
		return data[i];
	}
	inline const SpinImage& operator[]( int i ) const {
		return data[i];
	}

	void querySimilar( const SpinImage& query, idxList &result, const char *scoreOutput = NULL );
	void copyParams( const SpinImageStack& sis );

	real_type binSize;
	int imageWidth;
	real_type supportAngle;
	real_type lambda;				/**< Half of median bin size */

private:
	std::vector<SpinImage> data;
};

#endif
