// From the multiSelect example
#include <QGLViewer/qglviewer.h>
#include <QGLViewer/frame.h>
#include <QGLViewer/constraint.h>
#include "types.h"
#include "CostFunctions.h"

class TransformObject
{
public :
	void draw() const;
	qglviewer::Frame frame;

	real3_type position, normal;
};


class ManipulatedFrameSetConstraint : public qglviewer::Constraint
{
public:
	void clearSet();
	void addObjectToSet(TransformObject* o);

	virtual void constrainTranslation(qglviewer::Vec &translation, qglviewer::Frame *const frame);
	virtual void constrainRotation(qglviewer::Quaternion &rotation, qglviewer::Frame *const frame);

private :
	QList<TransformObject*> objects_;
};


class Scene;
class ShapeModel;
class SE3;

class TransformViewer : public QGLViewer
{
	Q_OBJECT

public slots:
	void evaluateCost();

public:
	TransformViewer( Scene *sc, ShapeModel *source, ShapeModel *target );
	virtual ~TransformViewer();

	std::vector<SE3>& getAssignedTf();
	void setAssignedTf( index_type i, const SE3& tf );
	std::vector<SE3> *assignedTf;

	DataFcnArray dataFunctions;
	SmoothFcnArray smoothFunctions;
	std::vector<QString> dataFunctionNames, smoothFunctionNames;

	data_type *dataCost, *smoothCost;

protected :
	virtual void draw();
	virtual void init();
	virtual QString helpString() const;

	// Selection functions
	virtual void drawWithNames();
	virtual void endSelection(const QPoint&);

	virtual void keyPressEvent(QKeyEvent *e);

	// Mouse events functions
	virtual void mousePressEvent(QMouseEvent *e);
	virtual void mouseMoveEvent(QMouseEvent *e);
	virtual void mouseReleaseEvent(QMouseEvent *e);

private :
	void startManipulation();
	void drawSelectionRectangle() const;
	void addIdToSelection(int id);
	void removeIdFromSelection(int id);
	void updateManipulatedFrameCenter();
	void computeRegistrationError();
	void drawRegistrationError();
	void drawColormap();

	// Shapes
	Scene *scene;
	ShapeModel *sourceShape, *targetShape;
	bool showSource, showTarget;
	bool useSameColor;

	// Compute registration error
	distList registrationError;
	bool showRegistrationError;

	//////////////////////////////////////////////////////////////////////////
	// Current rectangular selection
	QRect rectangle_;

	// Different selection modes
	enum SelectionMode { NONE, ADD, REMOVE };
	SelectionMode selectionMode_;

	QList<TransformObject*> objects_;
	QList<int> selection_;

};
